*===================================================================
*
* A procedure to slow down a GOLD CARD by introducing a delay. The
* delay parameter is also returned by a function.
*
* SLOW n - sets the delay factor (-32768 < n < 32767)
*          (masked to range 0 - 31 only)
* delay = HOW_SLOW - returns the current delay value (0 - 15)         
* GO_SLOW - turns delay procedures on.                                
* NO_SLOW - turns delay procedures off.                               
* running = SLOW_ON - returns 1 if running or 0 otherwise.            
*
*=====================================================================
*
*               Copyright Norman Dunbar  August 1991
*              Modified by Dilwyn Jones December 1991
*
*=====================================================================


*---------------------------------------------------------------------
*
* CODE. (To insert the new routine(s) into SuperBasic).               
*
*---------------------------------------------------------------------

call_here  move.w  $110,a2             ;Now link procs & functions
           lea     define,a1           ;Start address of definitions
           jmp     (a2)                ;Do it & return to Basic

linkage    ds.l    2                   ;Space for linkage block
speed      dc.w    0                   ;Current delay factor
running    dc.b    0                   ;Has GO_SLOW been called ?

*---------------------------------------------------------------------
*
* DATA. (Definition block for the new routine(s).                     
*
*---------------------------------------------------------------------

define     dc.w    3                   ;No of procedures
           dc.w    slow-*
           dc.b    4
           dc.b    'SLOW'

           dc.w    go_slow-*
           dc.b    7
           dc.b    'GO_SLOW'

           dc.w    no_slow-*
           dc.b    7
           dc.b    'NO_SLOW'
           dc.w    0                   ;End of procedures

           dc.w    2                   ;No of functions
           dc.w    how_slow-*
           dc.b    8
           dc.b    'HOW_SLOW'

           dc.w    slow_on-*
           dc.b    7
           dc.b    'SLOW_ON'
           dc.w    0                   ;End of functions

*---------------------------------------------------------------------
*
* Code for GO_SLOW procedure.                                         
*
*                                                                     
*
* Note the use of a flag byte, to ensure that the MT_LPOLL routine is 
* not called more than once while the polling routine is active, this 
* would cause the polled list to loop forever and never reach the end 
* thus causing the QL to hang. (QDOS Companion page 97)               
*
* Test the running flag and if not running set up the linkage block so
* that QDOS knows which routine to run every interrupt. Link in the   
* routine using MT_LPOLL and set the running flag.                    
*
*---------------------------------------------------------------------

go_slow    lea.l   running,a2          ;Flag byte address
           tas     (a2)            ;Check if running, and set 'running'
           bne.s   home                ;Currently running, exit
           lea.l   linkage,a0          ;Address of two long words
           lea.l   slow_down,a1        ;Delay routine
           move.l  a1,4(a0)            ;Store routine address
           moveq   #$1C,d0             ;Link polled interrupt
           trap    #1                  ;Do it (no errors)
           bra.s   home                ;Bale out with no errors

*---------------------------------------------------------------------
*
* Code for NO_SLOW procedure.                                         
*                                                                     
* Checks the flag byte first to ensure that a non-existant routine is 
* not removed. Assuming that the routine is running, unlink it from   
* the linked list and clear the running flag.                         
*
*---------------------------------------------------------------------


no_slow    lea.l   running,a2         ;Address of 'running' flag
           tst.b   (a2)               ;Check it
           beq.s   home               ;Not running, exit
           lea.l   linkage,a0         ;Address of two long words
           moveq   #$1D,d0            ;Unlink polled interrupt
           trap    #1                 ;Do it (no errors & preserves A2)
           clr.b   (a2)               ;Set 'not running'
           bra.s   home               ;Bale out with no errors

*---------------------------------------------------------------------
*
* Code for SLOW n procedure.                                          
*                                                                   
* Get one word sized integer from the stack, force it to be a positive
* number and in range 0 - 15, then store it wher the interrupt routine
* can find it.                                                        
*
*---------------------------------------------------------------------

slow       move.w  $112,a2             ;Word integer required
           jsr     (a2)                ;Get some
           bne.s   exit                ;Oops...
           subq.w  #1,d3               ;Should be only 1
           bne.s   bad_param           ;Oops...
           move.w  0(a6,a1.l),d1       ;Get the speed
           andi.w  #$001f,d1           ;Make positive & in range 0-31
           lea.l   speed,a0            ;Point to where it's going
           move.w  d1,(a0)             ;Stick it there

home       moveq   #0,d0               ;No errors
exit       rts                         ;Back to Basic

bad_param  moveq   #-15,d0             ;Bad parameter
           rts                         ;Back to Basic

*---------------------------------------------------------------------
*
* Code for HOW_SLOW function.                                         
*                                                                    
* Read the current speed from store, then make room for a word sized  
* integer on the maths stack, stack the result and return to Basic.   
*---------------------------------------------------------------------

how_slow   lea.l   speed,a0          ;Pointer to result
           move.w  (a0),d7           ;Store for now

ret_d7     moveq   #2,d1             ;Space for a word integer required
           move.w   $11A,a2          ;Prepare to get some space
           jsr     (a2)              ;Go get it (no errors)
           move.l  $58(a6),a1        ;Get new maths pointer
           subq.l  #2,a1             ;Make room
           move.w  d7,0(a6,a1.l)     ;Store result on maths stack
           move.l  a1,$58(a6)        ;Store updated maths pointer
           moveq   #3,d4             ;Result is word integer
           bra.s   home              ;Exit to Basic

*---------------------------------------------------------------------
*
* Code for the SLOW_ON function.                                      
*
* Read the running flag and if it is not zero return 1 else return 0. 
*
*---------------------------------------------------------------------


slow_on    moveq   #0,d7               ;Assume not running
           lea.l   running,a2          ;Get 'running' flag address
           move.b  (a2),d7             ;Get the flag
           neg.b   d7                  ;0 or 1
           bra.s   ret_d7              ;Return the result

*---------------------------------------------------------------------
*
* Code for the delay routine (50/60 Hz interrupt)                     
*
*---------------------------------------------------------------------

slow_down  lea.l   speed,a0      ;Point to current speed
           move.w  (a0),d1       ;Get current speed
           beq.s   quit          ;Zero means no delay
           bra.s   outer         ;Otherwise skip

inner      move.w  #1000,d2      ;Seems like a good delay !
again      nop                   ;Delay 4 clock cycles
           nop                   ;Twice !
           dbra    d2,again      ;And a few more times

outer      dbra    d1,inner      ;And once again
           moveq   #0,d0         ;Possibly not needed, but just in case

quit       rts                   ;Finished interrupting
